
crown = TRUE
df1 = 0.5 # m1 vs m2
df2 = 2 # m1 vs m3
degrees.of.freedom = 1.5 # m2 vs m3

####

# model 2 vs 3 only 
analysis<-function(tree1, frs, rho, name="name", file1 = "lk_ratio_test_results_empirical.txt", file2 = "lk_ratio_test_results_empirical.tex"){

  # rename range data headers
  names(frs)[2]<-"sp"
  names(frs)[3]<-"start"
  names(frs)[4]<-"end"

  ## model 2
  # constrained model with lambda, lambda.star and d (= d*)
  
  cat("lambda\t", "mu\t", "lambda*\t", "mu*\t", "lk\n", file=paste(name,".null.lk.log.txt",sep=""), append=F)

  lnL = -10^100
  tmp = NULL
  
  for(j in 1:20){
    
    attempts = 1
    convergence = 1
    
    while(convergence != 0){
      
      b = runif(1)
      b.star = runif(1, b, 2)
      nd = runif(1, 0, b*.99)
      
      est = fbdR::est.bd.combined(tree1, frs, b = b, b.star = b.star, nd = nd, constrained = TRUE, crown = crown, rho = rho)
      
      convergence = est$convergence
      
      if(attempts == 100)
        break
      
      attempts = attempts + 1
    }
    
    if(convergence != 0)
      next
    else {
      if(est$value > lnL){
        lnL = est$value
        tmp = est
      }
    }
    cat(est$par[1],"\t", est$par[1]-est$par[3], "\t", est$par[2], "\t", est$par[2]-est$par[3], "\t", est$value, "\n", file=paste(name,".null.lk.log.txt",sep=""), append=T)
  }
  
  if(is.null(tmp))
    stop("Something went wrong!")

  est = tmp
  birth.extant1 = est$par[1]
  birth.ranges1 = est$par[2]
  death.extant1 = est$par[1]-est$par[3]
  death.ranges1 = est$par[2]-est$par[3]
  convergence1 = est$convergence
  lk1 = est$value
  if(lk1 != lnL)
    print("Something went wrong!")

  ## model 3
  # unconstrained model with lambda, lambda.star, mu and mu.star
  
  cat("lambda\t", "mu\t", "lambda*\t", "mu*\t", "lk\n", file=paste(name,".alt.lk.log.txt",sep=""), append=F)

  lnL = -10^100
  tmp = NULL
  
  for(j in 1:20){
    
    attempts = 1
    convergence = 1
    
    while(convergence != 0){
      
      b = runif(1)
      d = runif(1, 0, b*.99)
      b.star = runif(1, b, 2)
      d.star = runif(1, 0, b.star*.99)
      
      est = fbdR::est.bd.combined(tree1, frs, b = b, d = d, b.star = b.star, d.star = d.star, constrained = FALSE, crown = crown, rho = rho)
      
      convergence = est$convergence
      
      if(attempts == 100)
        break
      attempts = attempts + 1
      
    }
    if(convergence != 0)
      next
    if(est$value > lnL){
      lnL = est$value
      tmp = est
    }
    cat(est$par[1], "\t", est$par[2], "\t", est$par[3], "\t", est$par[4], "\t", est$value, "\n", file=paste(name,".alt.lk.log.txt",sep=""), append=TRUE)
  }
  
  if(is.null(tmp))
    stop("Something went wrong!")

  est = tmp
  birth.extant2 = est$par[1]
  death.extant2 = est$par[2]
  birth.ranges2 = est$par[3]
  death.ranges2 = est$par[4]
  convergence2 = est$convergence
  lk2 = est$value
  if(lk2 != lnL)
    print("Something went wrong!")

  # likelihood ratio test, degrees of freedom = 1
  LR = 2 * (lk2 - lk1)

  # accept or reject the null model
  # reject the alternative if:
  if(LR > qchisq(.95, df=degrees.of.freedom))
    accept1 = "reject"
  else
    accept1 = "accept"

  # accept or reject the null model
  # reject the alternative if:
  if(LR > qchisq(.99, df=degrees.of.freedom))
    accept2 = "reject"
  else
    accept2 = "accept"

  # collect interesting info
  r = length(frs$sp)
  mx = round(max(frs$start),1)
  t = length(tree1$tip.label)
  root.age = round(max(TreeSim::getx(tree1)),1)

  # print out interesting info
  cat(name, "\t",
      r,"\t",
      mx,"\t",
      t, "\t",
      root.age,"\t",
      rho,"\t",
      # model 1
      round(birth.extant1,3),"\t",
      round(death.extant1,3),"\t",
      round(birth.ranges1,3),"\t",
      round(death.ranges1,3),"\t",
      round(birth.extant1 - death.extant1,3),"\t",
      round(birth.ranges1 - death.ranges1,3),"\t",
      # model 2
      round(birth.extant2,3),"\t",
      round(death.extant2,3),"\t",
      round(birth.ranges2,3),"\t",
      round(death.ranges2,3),"\t",
      round(birth.extant2 - death.extant2,3),"\t",
      round(birth.ranges2 - death.ranges2,3),"\t",
      round(lk1,3),"\t",
      round(lk2,3),"\t",
      round(LR,3),"\t",
      accept1,"\t",
      accept2,"\n",
      sep = "",
      file = file1,
      append = TRUE
  )

  # cat(name, " & ",
  #     r, " & ",
  #     mx, " & ",
  #     t, " & ",
  #     root.age," & ",
  #     rho," & ",
  #     round(birth.extant1,3)," & ",
  #     round(birth.extant2,3)," & ",
  #     round(death.extant1,3)," & ",
  #     round(death.extant2,3)," & ",
  #     round(birth.ranges1,3)," & ",
  #     round(birth.ranges2,3)," & ",
  #     round(death.ranges1,3)," & ",
  #     round(death.ranges2,3)," & ",
  #     round(birth.extant1 - death.extant1,3)," & ",
  #     round(birth.extant2 - death.extant2,3)," & ",
  #     round(birth.ranges1 - death.ranges1,3)," & ",
  #     round(birth.ranges2 - death.ranges2,3)," & ",
  #     round(lk1,3)," & ",
  #     round(lk2,3)," & ",
  #     round(LR,3)," & ",
  #     accept1," & ",
  #     accept2," \\\\\n",
  #     sep = "",
  #     file = file2,
  #     append = TRUE
  # )

  cat("Outcome of LR test for ", name, " = ", accept1, " and ", accept2, "\n", file = "log.txt", append=T, sep = "")

  #eof
}

print.headers<-function(file1 = "lk_ratio_test_results_empirical.txt", file2 = "lk_ratio_test_results_empirical.tex"){

  cat("Clade", "\t",
      "ranges","\t",
      "max_ranges","\t",
      "tips", "\t",
      "root_age","\t",
      "rho","\t",
      "lambda_null","\t",
      "mu_null","\t",
      "lambda*_null","\t",
      "mu*_null", "\t",
      "d_null","\t",
      "d*_null","\t",
      "lambda_alt","\t",
      "mu_alt","\t",
      "lambda*_alt","\t",
      "mu*_alt", "\t",
      "d_alt","\t",
      "d*_alt","\t",
      "lk_null","\t",
      "lk_alt","\t",
      "LR","\t",
      "Outcome_.95","\t",
      "Outcome_.99","\n",
      sep = "",
      file = file1,
      append = F
  )

  # cat("Clade", " & ",
  #     "ranges", " & ",
  #     "max_ranges", " & ",
  #     "tips", " & ",
  #     "root_age", " & ",
  #     "rho", " & ",
  #     "lambda_null", " & ",
  #     "lambda_alt", " & ",
  #     "mu_null", " & ",
  #     "mu_alt", " & ",
  #     "lambda*_null", " & ",
  #     "lambda*_alt", " & ",
  #     "mu*_null", " & ",
  #     "mu*_alt", " & ",
  #     "d_null", " & ",
  #     "d_alt", " & ",
  #     "d*_null", " & ",
  #     "d*_alt", " & ",
  #     "lk_null", " & ",
  #     "lk_alt", " & ",
  #     "LR", " & ",
  #     "Outcome._95", " & ",
  #     "Outcome._99"," \\\\\n",
  #     sep = "",
  #     file = file2,
  #     append = F
  # )
  
  #eof
}

####

analysis.3models<-function(tree1, frs, rho, name="name", file1 = "lk_ratio_test_results_empirical_3m.txt", file2 = "lk_ratio_test_results_empirical_3m.tex"){

  # rename range data headers
  names(frs)[2]<-"sp"
  names(frs)[3]<-"start"
  names(frs)[4]<-"end"

  # model 1
  # constrained model with lambda = lambda.star and mu = mu*
  
  cat("lambda\t", "mu\t", "lambda*\t", "mu*\t", "lk\n", file=paste(name,".m1.lk.log.txt",sep=""), append=F)

  lnL = -10^100
  tmp = NULL
  
  for(j in 1:20){
    
    attempts = 1
    convergence = 1
    
    while(convergence != 0){
      
      d = runif(1)
      b = runif(1, d, 2)
      
      est = fbdR::est.bd.combined(tree1, frs, b = b, d = d, constrained = TRUE, crown = crown, rho = rho, constrained.p = 2)
      
      convergence = est$convergence
      
      if(attempts == 100)
        break
      attempts = attempts + 1
    }
    
    if(convergence != 0)
      next
    else{
      if (est$value > lnL){
        lnL = est$value
        tmp = est
      }
    }
    cat(est$par[1],"\t", est$par[2], "\t", est$par[1], "\t", est$par[2], "\t", est$value, "\n", file=paste(name,".m1.lk.log.txt",sep=""), append=T)
  }
  
  if(is.null(tmp))
    stop("Something went wrong!")

  est = tmp
  birth.extant1 = est$par[1]
  birth.ranges1 = est$par[1]
  death.extant1 = est$par[2]
  death.ranges1 = est$par[2]
  convergence1 = est$convergence
  lk1 = est$value
  if(lk1 != lnL)
    print("Something went wrong!")

  # model 2
  # constrained model with lambda, lambda.star and d (= d*)
  
  cat("lambda\t", "mu\t", "lambda*\t", "mu*\t", "lk\n", file=paste(name,".m2.lk.log.txt",sep=""), append=F)

  lnL = -10^100
  tmp = NULL
  
  for(j in 1:20){
    
    attempts = 1
    convergence = 1
    
    while(convergence != 0){
      
      b = runif(1)
      b.star = runif(1, b, 2)
      nd = runif(1, 0, b*.99)
      
      est = fbdR::est.bd.combined(tree1, frs, b = b, b.star = b.star, nd = nd, constrained = TRUE, crown = crown, rho = rho)
      
      convergence = est$convergence
      
      if(attempts == 100)
        break
      attempts = attempts + 1
    }
    if(convergence != 0)
      next
    else {
      if(est$value > lnL){
        lnL = est$value
        tmp = est
      }
    }
    cat(est$par[1],"\t", est$par[1]-est$par[3], "\t", est$par[2], "\t", est$par[2]-est$par[3], "\t", est$value, "\n", file=paste(name,".m2.lk.log.txt",sep=""), append=T)
  }
  
  if(is.null(tmp))
    stop("Something went wrong!")

  est = tmp
  birth.extant2 = est$par[1]
  birth.ranges2 = est$par[2]
  death.extant2 = est$par[1]-est$par[3]
  death.ranges2 = est$par[2]-est$par[3]
  convergence2 = est$convergence
  lk2 = est$value
  if(lk2 != lnL)
    print("Something went wrong!")

  # model 3
  # unconstrained model with lambda, lambda.star, mu and mu.star
  
  cat("lambda\t", "mu\t", "lambda*\t", "mu*\t", "lk\n", file=paste(name,".m3.lk.log.txt",sep=""), append=F)
  
  lnL = -10^100
  tmp = NULL
  
  for(j in 1:20){
    
    attempts = 1
    convergence = 1
    
    while(convergence != 0){
      
      b = runif(1)
      d = runif(1, 0, b*.99)
      b.star = runif(1, b, 2)
      d.star = runif(1, 0, b.star*.99)
      
      est = fbdR::est.bd.combined(tree1, frs, b = b, d = d, b.star = b.star, d.star = d.star, constrained = FALSE, crown = crown, rho = rho)
      convergence = est$convergence
      
      if(attempts == 100)
        break
      attempts = attempts + 1
    }
    
    if(convergence != 0)
      next
    else{
      if(est$value > lnL){
        lnL = est$value
        tmp = est
      }
    }
    cat(est$par[1], "\t", est$par[2], "\t", est$par[3], "\t", est$par[4], "\t", est$value, "\n", file=paste(name,".m3.lk.log.txt",sep=""), append=TRUE)
  }
  
  if(is.null(tmp))
    stop("Something went wrong!")

  est = tmp
  birth.extant3 = est$par[1]
  death.extant3 = est$par[2]
  birth.ranges3 = est$par[3]
  death.ranges3 = est$par[4]
  convergence3 = est$convergence
  lk3 = est$value
  if(lk3 != lnL)
    print("Something went wrong!")

  ## model 1 vs 2
  # likelihood ratio test, degrees of freedom = 0.5
  LR1 = 2 * (lk2 - lk1)

  # accept or reject the null model
  # reject the alternative if:
  if(LR1 > qchisq(.95, df=df1))
    a1 = "reject"
  else
    a1 = "accept"

  # accept or reject the null model
  # reject the alternative if:
  if(LR1 > qchisq(.99, df=df1))
    a2 = "reject"
  else
    a2 = "accept"

  ## model 1 vs 3
  # likelihood ratio test, degrees of freedom = 2
  LR2 = 2 * (lk3 - lk1)
  
  # accept or reject the null model
  # reject the alternative if:
  if(LR2 > qchisq(.95, df=df2))
    a3 = "reject"
  else
    a3 = "accept"
  
  # accept or reject the null model
  # reject the alternative if:
  if(LR2 > qchisq(.99, df=df2))
    a4 = "reject"
  else
    a4 = "accept"
  
  ## model 2 vs 3
  # likelihood ratio test, degrees of freedom = 1.5
  LR3 = 2 * (lk3 - lk2)

  # accept or reject the null model
  # reject the alternative if:
  if(LR3 > qchisq(.95, df=degrees.of.freedom))
    a5 = "reject"
  else
    a5 = "accept"

  # accept or reject the null model
  # reject the alternative if:
  if(LR3 > qchisq(.99, df=degrees.of.freedom))
    a6 = "reject"
  else
    a6 = "accept"
  
  # collect interesting info
  r = length(frs$sp)
  mx = round(max(frs$start),1)
  t = length(tree1$tip.label)
  root.age = round(max(TreeSim::getx(tree1)),1)

  # print out interesting info
  cat(name, "\t",
      r,"\t",
      mx,"\t",
      t, "\t",
      root.age,"\t",
      rho,"\t",
      # model 1
      round(birth.extant1,3),"\t",
      round(death.extant1,3),"\t",
      round(birth.ranges1,3),"\t",
      round(death.ranges1,3),"\t",
      round(birth.extant1 - death.extant1,3),"\t",
      round(birth.ranges1 - death.ranges1,3),"\t",
      # model 2
      round(birth.extant2,3),"\t",
      round(death.extant2,3),"\t",
      round(birth.ranges2,3),"\t",
      round(death.ranges2,3),"\t",
      round(birth.extant2 - death.extant2,3),"\t",
      round(birth.ranges2 - death.ranges2,3),"\t",
      # model 3
      round(birth.extant3,3),"\t",
      round(death.extant3,3),"\t",
      round(birth.ranges3,3),"\t",
      round(death.ranges3,3),"\t",
      round(birth.extant3 - death.extant3,3),"\t",
      round(birth.ranges3 - death.ranges3,3),"\t",
      # likelihood
      round(lk1,3),"\t",
      round(lk2,3),"\t",
      round(lk3,3),"\t",
      # m1 vs m2
      round(LR1,3),"\t",
      a1,"\t",
      a2,"\t",
      # m1 vs m3
      round(LR2,3),"\t",
      a3,"\t",
      a4,"\t",
      # m2 vs m3
      round(LR3,3),"\t",
      a5,"\t",
      a6,"\n",
      sep = "",
      file = file1,
      append = TRUE
  )

  cat("Outcome of LR1 test for ", name, " = ", a1, " and ", a2, "\n", file = "log.txt", append=T, sep = "")
  cat("Outcome of LR3 test for ", name, " = ", a5, " and ", a6, "\n", file = "log.txt", append=T, sep = "")

  #eof
}

print.headers.3models<-function(file1 = "lk_ratio_test_results_empirical_3m.txt", file2 = "lk_ratio_test_results_empirical_3m.tex"){

  cat("Clade", "\t",
      "ranges","\t",
      "max_ranges","\t",
      "tips", "\t",
      "root_age","\t",
      "rho","\t",
      # model 1
      "lambda_1","\t",
      "mu_1","\t",
      "lambda*_1","\t",
      "mu*_1", "\t",
      "d_1","\t",
      "d*_1","\t",
      # model 2
      "lambda_2","\t",
      "mu_2","\t",
      "lambda*_2","\t",
      "mu*_2", "\t",
      "d_2","\t",
      "d*_2","\t",
      # model 3
      "lambda_3","\t",
      "mu_3","\t",
      "lambda*_3","\t",
      "mu*_3", "\t",
      "d_3","\t",
      "d*_3","\t",
      # lk estimates
      "lk_1","\t",
      "lk_2","\t",
      "lk_3","\t",
      # m1 vs m2
      "LR1","\t",
      "Outcome_.95","\t",
      "Outcome_.99","\t",
      # m1 vs m3
      "LR2","\t",
      "Outcome_.95","\t",
      "Outcome_.99","\t",
      # m2 vs m3
      "LR3","\t",
      "Outcome_.95","\t",
      "Outcome_.99","\n",
      sep = "",
      file = file1,
      append = F
  )

  #eof
}
